@@ -87,7 +87,7 @@ def run_script_from_file():
 global loaded_filename
 
 def open_xlsx_file(*args): 
-    table=args[0]
+    table1=args[0]
     file = askopenfile(mode ='r', filetypes =[('Excel files', '*.xlsx')]) 
     filename=""
     if file is not None: 
@@ -265,8 +265,7 @@ class Table:
         
             
             
-    def is_clicked(self,pos):
-        print("bla")
+    def is_point_in_rectangle(self,pos):
         if self.pos[0]<pos[0] and pos[0]<self.pos[0]+self.table_size[0] and self.pos[1]<pos[1] and pos[1]<self.pos[1]+self.table_size[1]:
             return(True)
         else:
@@ -496,16 +495,18 @@ class TextArea(DraggableRect):
 
 
 class Button(DraggableRect):
-    def __init__(self,pos,size,text,function=lambda *args:None,border_color=(0,0,0),color=(200,200,200)):
+    def __init__(self,pos,size,text,function=lambda *args:None,function_args=None,border_color=(0,0,0),color=(200,200,200),relative_pos=[0,0]):
         super().__init__(pos,size,color,draggable=False)
         self.pos=pos
         self.size=size
         self.border_color=border_color
         self.color=color
         self.text=text
-        self.label=Label(self.text,(0,0,0),[pos[0]+2,pos[1]+4],font_type="Calibri",font_size=15,max_text_length=size[0]-1)
+        self.label=Label(self.text,(0,0,0),[pos[0]+2,pos[1]+4],font_type="Calibri",font_size=15,max_text_length=size[0]-1,relative_pos=[2,4])
         self.function=function
         self.is_clicked=False
+        self.function_args=function_args
+        self.relative_pos=relative_pos
         
     def draw(self):
         if self.is_clicked:
@@ -518,8 +519,13 @@ class Button(DraggableRect):
     def run_function(self,*args):
         if len(args)>0:
             self.function(args)
+        elif self.function_args is not None:
+            self.function(self.function_args)
         else:
             self.function()
+            
+    def on_click(self):
+        self.run_function()
         
 
     
@@ -527,34 +533,56 @@ class Button(DraggableRect):
 
 
 class ComboBox(DraggableRect):
-    def __init__(self, pos, size, values, text=None, border_color=(0,0,0)):
+    def __init__(self, pos, size, values, text=None, border_color=(0,0,0),relative_pos=[0,0]):
         super().__init__(pos, size, color, draggable=False)
         self.pos = pos
         self.size = size
         self.values = values
-        self.text = text
+        #self.text = text
         self.border_color = border_color
         self._labels = []
         self._is_rolled = False
+        self.visible=True
+        self.relative_pos=relative_pos
         if text:
             self.cell = Cell(self.pos, self.size, (212,212,212), coor=[0, 0])
             self.cell.label.text = text
             self.cell.label.pos[0]=self.cell.label.pos[0]+1 #2 pixels from border
+        #TODO elif - create empty Cell
 
     def draw(self):
-        if hasattr(self, "text"):
-            pygame.draw.rect(screen,self.color,self.cell.pos+self.cell.size)    
-            self.cell.draw()         
-            pygame.draw.rect(screen,(0,0,0),self.cell.pos+self.cell.size,2)
-            pygame.draw.line(screen,(0,0,0),[self.pos[0]+self.size[0]-15,self.pos[1]+self.size[1]/2],[self.pos[0]+self.size[0]-10,self.pos[1]+self.size[1]/2+5],3)
-            pygame.draw.line(screen,(0,0,0),[self.pos[0]+self.size[0]-10,self.pos[1]+self.size[1]/2+5],[self.pos[0]+self.size[0]-5,self.pos[1]+self.size[1]/2],3)
-            pygame.draw.line(screen,(0,0,0),[self.pos[0]+self.size[0]-18,self.pos[1]],[self.pos[0]+self.size[0]-18,self.pos[1]+self.size[1]],1)
-            
-        if self._is_rolled:
-            for label, value in zip(self._labels, self.values):
-                label.label.text = value
-                label.draw()
+        if self.visible:
+            if hasattr(self, "cell"):
+                pygame.draw.rect(screen,self.color,self.cell.pos+self.cell.size)    
+                self.cell.draw()         
+                pygame.draw.rect(screen,(0,0,0),self.cell.pos+self.cell.size,2)
+                pygame.draw.line(screen,(0,0,0),[self.pos[0]+self.size[0]-15,self.pos[1]+self.size[1]/2],[self.pos[0]+self.size[0]-10,self.pos[1]+self.size[1]/2+5],3)
+                pygame.draw.line(screen,(0,0,0),[self.pos[0]+self.size[0]-10,self.pos[1]+self.size[1]/2+5],[self.pos[0]+self.size[0]-5,self.pos[1]+self.size[1]/2],3)
+                pygame.draw.line(screen,(0,0,0),[self.pos[0]+self.size[0]-18,self.pos[1]],[self.pos[0]+self.size[0]-18,self.pos[1]+self.size[1]],1)
                 
+            if self._is_rolled:
+                for label, value in zip(self._labels, self.values):
+                    label.label.text = value
+                    label.draw()
+                
+    def is_point_in_rectangle(self,pos):
+        if not self._is_rolled:
+            result=super().is_point_in_rectangle(pos)
+            return(result)
+        else:
+            if self.pos[0]<pos[0] and pos[0]<self.pos[0]+self.size[0] and self.pos[1] < pos[1] and pos[1] < self.pos[1] + self.size[1] * (len(self.values) + 1):
+                return(True)
+            else:
+                return(False)  
+                    
+    def on_click(self):
+        pos=pygame.mouse.get_pos()        
+        if self._is_rolled and self.pos[1] < pos[1] and pos[1] < self.pos[1] + self.size[1] * (len(self.values) + 1):
+            self.choose(pos)
+        elif not self._is_rolled and self.is_point_in_rectangle(pos):
+            self.choose(pos)
+        
+        #self.choose(pos)
 
     def roll(self):
         if not self._is_rolled:
@@ -633,11 +661,8 @@ def main_program_loop(pgwidgets,table1):
                                         pgwidgets[i].elements[j].is_clicked=True
                                         
                                 if type(widget)==ComboBox:
-                                    if widget._is_rolled and widget.pos[1] < pos[1] and pos[1] < widget.pos[1] + widget.size[1] * (len(widget.values) + 1):
-                                        widget.choose(pos)
-                                    elif not widget._is_rolled and widget.is_point_in_rectangle(pos):
-                                        widget.choose(pos)
-                            
+                                    widget.on_click()
+                                    
                             
                         
                                 
